@@ -142,7 +142,7 @@ def finisher(once=False, sleep_time=60, activities=None, bulk=100, db_bulk=1000,
                         time3 = time.time()
                         __handle_requests(chunk, suspicious_patterns, retry_protocol_mismatches, logger=logger)
                         record_timer('daemons.conveyor.finisher.handle_requests', (time.time() - time3) * 1000 / (len(chunk) if chunk else 1))
-                        record_counter('daemons.conveyor.finisher.handle_requests', len(chunk))
+                        record_counter('daemons.conveyor.finisher.handle_requests', delta=len(chunk))
                     except Exception as error:
                         logger(logging.WARNING, '%s', str(error))
                 if reqs:
