@@ -126,13 +126,20 @@ def poller(once=False, activities=None, sleep_time=60,
                                                 limit=db_bulk,
                                                 older_than=datetime.datetime.utcnow() - datetime.timedelta(seconds=older_than),
                                                 total_workers=heart_beat['nr_threads'], worker_number=heart_beat['assign_thread'],
-                                                mode_all=False, hash_variable='id',
+                                                mode_all=True, hash_variable='id',
                                                 activity=activity,
                                                 activity_shares=activity_shares,
                                                 transfertool=FILTER_TRANSFERTOOL)
 
                 record_timer('daemons.conveyor.poller.000-get_next', (time.time() - start_time) * 1000)
 
+                if TRANSFER_TOOL and not FILTER_TRANSFERTOOL:
+                    # only keep transfers which don't have any transfertool set, or have one equal to TRANSFER_TOOL
+                    transfs_tmp = [t for t in transfs if not t['transfertool'] or t['transfertool'] == TRANSFER_TOOL]
+                    if len(transfs_tmp) != len(transfs):
+                        logger(logging.INFO, 'Skipping %i transfers because of missmatched transfertool', len(transfs) - len(transfs_tmp))
+                    transfs = transfs_tmp
+
                 if transfs:
                     logger(logging.DEBUG, 'Polling %i transfers for activity %s' % (len(transfs), activity))
 
