@@ -20,17 +20,19 @@
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
+# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 from __future__ import print_function
 
 import base64
-import imp
 import logging
 import time
 from re import search
 from typing import TYPE_CHECKING
 
 from flask import Flask, Blueprint, request, Response, redirect, render_template
+from six.moves.urllib.parse import urlparse
 from werkzeug.datastructures import Headers
 
 from rucio.api.authentication import get_auth_token_user_pass, get_auth_token_gss, get_auth_token_x509, \
@@ -38,29 +40,20 @@ from rucio.api.authentication import get_auth_token_user_pass, get_auth_token_gs
     get_token_oidc, refresh_cli_auth_token, get_auth_token_saml
 from rucio.common.config import config_get
 from rucio.common.exception import AccessDenied, IdentityError, CannotAuthenticate, CannotAuthorize
-from rucio.common.utils import date_to_str, urlparse
-from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, generate_http_error_flask, \
-    ErrorHandlingMethodView
-from rucio.web.rest.utils import error_headers
+from rucio.common.extra import import_extras
+from rucio.common.utils import date_to_str
+from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, error_headers, \
+    extract_vo, generate_http_error_flask, ErrorHandlingMethodView
 
 if TYPE_CHECKING:
     from typing import Optional
     from rucio.web.rest.flaskapi.v1.common import HeadersType
 
-# Extra modules: Only imported if available
-EXTRA_MODULES = {'onelogin': False}
-
-for extra_module in EXTRA_MODULES:
-    try:
-        imp.find_module(extra_module)
-        EXTRA_MODULES[extra_module] = True
-    except ImportError:
-        EXTRA_MODULES[extra_module] = False
+EXTRA_MODULES = import_extras(['onelogin'])
 
 if EXTRA_MODULES['onelogin']:
-    from onelogin.saml2.auth import OneLogin_Saml2_Auth
-    from web import cookies
-    from rucio.web.ui.common.utils import prepare_saml_request
+    from onelogin.saml2.auth import OneLogin_Saml2_Auth  # pylint: disable=import-error
+    from rucio.web.ui.flask.common.utils import prepare_saml_request
 
 
 class UserPass(ErrorHandlingMethodView):
@@ -119,7 +112,7 @@ class UserPass(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers['Pragma'] = 'no-cache'
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         username = request.headers.get('X-Rucio-Username', default=None)
         password = request.headers.get('X-Rucio-Password', default=None)
@@ -137,8 +130,8 @@ class UserPass(ErrorHandlingMethodView):
         if not result:
             return generate_http_error_flask(401, CannotAuthenticate.__name__, f'Cannot authenticate to account {account} with given credentials', headers=headers)
 
-        headers['X-Rucio-Auth-Token'] = result.token
-        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
+        headers['X-Rucio-Auth-Token'] = result['token']
+        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result['expires_at'])
         return '', 200, headers
 
 
@@ -182,7 +175,7 @@ class OIDC(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers.set('Pragma', 'no-cache')
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.environ.get('HTTP_X_RUCIO_ACCOUNT', 'webui')
         auth_scope = request.environ.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_SCOPE', "")
         audience = request.environ.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_AUDIENCE', "")
@@ -412,8 +405,8 @@ class TokenOIDC(ErrorHandlingMethodView):
         if not result:
             return generate_http_error_flask(401, CannotAuthorize.__name__, 'Cannot authorize token request.', headers=headers)
         if 'token' in result and 'webhome' not in result:
-            headers.set('X-Rucio-Auth-Token', result['token'].token)
-            headers.set('X-Rucio-Auth-Token-Expires', date_to_str(result['token'].expired_at))
+            headers.set('X-Rucio-Auth-Token', result['token']['token'])
+            headers.set('X-Rucio-Auth-Token-Expires', date_to_str(result['token']['expires_at']))
             return '', 200, headers
         elif 'webhome' in result:
             webhome = result['webhome']
@@ -422,10 +415,10 @@ class TokenOIDC(ErrorHandlingMethodView):
                 headers.set('Content-Type', 'text/html')
                 return render_template('auth_crash.html', crashtype='unknown_identity'), 401, headers
             # domain setting is necessary so that the token gets distributed also to the webui server
-            domain = '.'.join(urlparse.urlparse(webhome).netloc.split('.')[1:])
+            domain = '.'.join(urlparse(webhome).netloc.split('.')[1:])
             response = redirect(webhome, code=303)
             response.headers.extend(headers)
-            response.set_cookie('x-rucio-auth-token', value=result['token'].token, domain=domain, path='/')
+            response.set_cookie('x-rucio-auth-token', value=result['token']['token'], domain=domain, path='/')
             response.set_cookie('rucio-auth-token-created-at', value=str(time.time()), domain=domain, path='/')
             return response
         else:
@@ -473,7 +466,7 @@ class RefreshOIDC(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers.set('Pragma', 'no-cache')
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         token = request.headers.get('X-Rucio-Auth-Token', default=None)
         if token is None or account is None:
@@ -544,7 +537,7 @@ class GSS(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers['Pragma'] = 'no-cache'
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         gsscred = request.environ.get('REMOTE_USER')
         appid = request.headers.get('X-Rucio-AppID', default='unknown')
@@ -568,8 +561,8 @@ class GSS(ErrorHandlingMethodView):
                 headers=headers
             )
 
-        headers['X-Rucio-Auth-Token'] = result.token
-        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
+        headers['X-Rucio-Auth-Token'] = result['token']
+        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result['expires_at'])
         return '', 200, headers
 
 
@@ -623,7 +616,7 @@ class x509(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers['Pragma'] = 'no-cache'
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         dn = request.environ.get('SSL_CLIENT_S_DN')
         if not dn:
@@ -674,8 +667,8 @@ class x509(ErrorHandlingMethodView):
                 headers=headers
             )
 
-        headers['X-Rucio-Auth-Token'] = result.token
-        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
+        headers['X-Rucio-Auth-Token'] = result['token']
+        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result['expires_at'])
         return '', 200, headers
 
 
@@ -729,7 +722,7 @@ class SSH(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers['Pragma'] = 'no-cache'
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         signature = request.headers.get('X-Rucio-SSH-Signature', default=None)
         appid = request.headers.get('X-Rucio-AppID', default='unknown')
@@ -765,8 +758,8 @@ class SSH(ErrorHandlingMethodView):
                 headers=headers
             )
 
-        headers['X-Rucio-Auth-Token'] = result.token
-        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
+        headers['X-Rucio-Auth-Token'] = result['token']
+        headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result['expires_at'])
         return '', 200, headers
 
 
@@ -820,7 +813,7 @@ class SSHChallengeToken(ErrorHandlingMethodView):
         headers.add('Cache-Control', 'post-check=0, pre-check=0')
         headers['Pragma'] = 'no-cache'
 
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         appid = request.headers.get('X-Rucio-AppID', default='unknown')
         ip = request.headers.get('X-Forwarded-For', default=request.remote_addr)
@@ -835,8 +828,8 @@ class SSHChallengeToken(ErrorHandlingMethodView):
                 headers=headers
             )
 
-        headers['X-Rucio-SSH-Challenge-Token'] = result.token
-        headers['X-Rucio-SSH-Challenge-Token-Expires'] = date_to_str(result.expired_at)
+        headers['X-Rucio-SSH-Challenge-Token'] = result['token']
+        headers['X-Rucio-SSH-Challenge-Token-Expires'] = date_to_str(result['expires_at'])
         return '', 200, headers
 
 
@@ -886,8 +879,8 @@ class SAML(ErrorHandlingMethodView):
         if not EXTRA_MODULES['onelogin']:
             return "SAML not configured on the server side.", 400, headers
 
-        saml_nameid = cookies().get('saml-nameid')
-        vo = request.headers.get('X-Rucio-VO', default='def')
+        saml_nameid = request.cookies.get('saml-nameid', default=None)
+        vo = extract_vo(request.headers)
         account = request.headers.get('X-Rucio-Account', default=None)
         appid = request.headers.get('X-Rucio-AppID', default='unknown')
         ip = request.headers.get('X-Forwarded-For', default=request.remote_addr)
@@ -911,8 +904,8 @@ class SAML(ErrorHandlingMethodView):
                     headers=headers
                 )
 
-            headers.set('X-Rucio-Auth-Token', result.token)
-            headers.set('X-Rucio-Auth-Token-Expires', date_to_str(result.expired_at))
+            headers.set('X-Rucio-Auth-Token', result['token'])
+            headers.set('X-Rucio-Auth-Token-Expires', date_to_str(result['expires_at']))
             return '', 200, headers
 
         # Path to the SAML config folder
